/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.1 Ganglinienprognose
 * Copyright (C) 2019 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.glprog.glprog;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * Verwalter für zyklisch auszuführende Prognosen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class ZyklischeAnfragenBearbeiter {

    private static class ZyklischeAnfrage implements Runnable {

        private final AnfrageInBearbeitung anfrage;
        private ZyklischeAnfragenBearbeiter zyklischeAnfragenBearbeiter;

        ZyklischeAnfrage(ZyklischeAnfragenBearbeiter zyklischeAnfragenBearbeiter, AnfrageInBearbeitung anfrage) {
            this.zyklischeAnfragenBearbeiter = zyklischeAnfragenBearbeiter;
            this.anfrage = anfrage;
        }

        @Override
        public void run() {
            if (anfrage.zyklischePrognoseAusfuehren()) {
                zyklischeAnfragenBearbeiter.addAnfrage(anfrage);
            }
        }

    }

    private final ScheduledExecutorService executor = Executors.newScheduledThreadPool(10);

    public void addAnfrage(AnfrageInBearbeitung anfrage) {
        executor.schedule(new ZyklischeAnfrage(this, anfrage), 1, TimeUnit.SECONDS);
    }
}
