/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.1 Ganglinienprognose
 * Copyright (C) 2019 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.glprog.glprog.portals;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.NumberArray;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.iav.gllib.gllib.domain.ApproximationTyp;
import de.bsvrz.iav.gllib.gllib.domain.EreignisTyp;
import de.bsvrz.iav.gllib.gllib.domain.GanglinieAnfrageTyp;
import de.bsvrz.iav.gllib.gllib.domain.GanglinieTyp;
import de.bsvrz.iav.gllib.gllib.domain.MessQuerschnitt;
import de.bsvrz.iav.gllib.gllib.domain.Messwerte;
import de.bsvrz.iav.gllib.gllib.domain.MqGanglinie;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlAktualisierenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlAnlegenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlLesenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlLoeschenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlSchreibenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlSpeicherAnfrage;
import de.bsvrz.iav.gllib.gllib.portals.DavObjektFactory;
import de.bsvrz.iav.gllib.gllib.shared.Event;
import de.bsvrz.iav.glprog.glprog.domain.comm.GanglinieAnfrageEmpfaenger;
import de.bsvrz.iav.glprog.glprog.domain.comm.GlUngueltigeAnfrage;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Implementierung eines {@link GanglinieAnfrageEmpfaenger} mit Kopplung zu
 * einem Datenverteiler-System.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavGanglinienAnfrageEmpfaenger implements GanglinieAnfrageEmpfaenger {

	private static final Debug LOGGER = Debug.getLogger();

	private final ClientReceiverInterface receiver = new ClientReceiverInterface() {

		@Override
		public void update(ResultData[] results) {
			for (ResultData result : results) {
				if (result.hasData()) {
					LOGGER.fine(() -> "Anfrage: " + result);
					versendeAnfrage(result.getData());
				}
			}
		}
	};

	private final Event<GlSpeicherAnfrage> anfrageEmpfangen = new Event<>();

	private DavObjektFactory objektFactory;

	public DavGanglinienAnfrageEmpfaenger(DavObjektFactory objektFactory, SystemObject speicherObjekt) {
		this.objektFactory = objektFactory;

		ClientDavInterface dav = objektFactory.getDav();
		AttributeGroup atg = dav.getDataModel().getAttributeGroup("atg.ganglinienSpeicherAnfrage");
		Aspect asp = dav.getDataModel().getAspect("asp.anfrage");
		dav.subscribeReceiver(receiver, speicherObjekt, new DataDescription(atg, asp), ReceiveOptions.normal(),
				ReceiverRole.drain());
	}

	@Override
	public Event<GlSpeicherAnfrage> anfrageEmpfangen() {
		return anfrageEmpfangen;
	}

	private void versendeAnfrage(Data data) {
		anfrageEmpfangen.send(konvertiere(data));
	}

	private GlSpeicherAnfrage konvertiere(Data data) {

		try {
			GanglinieAnfrageTyp anfrageKommando = GanglinieAnfrageTyp
					.with(data.getItem("Anfrage").getUnscaledValue("KommandoAnfrage").intValue());

			switch (anfrageKommando) {
			case SCHREIBEN:
				if (data.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien").getLength() > 0) {
					return erzeugeSchreibenAnfrage(data);
				}
				return erzeugeLesenAnfrage(data);

			case LOESCHEN:
				return erzeugeLoeschenAnfrage(data);

			case AKTUALISIEREN:
				return erzeugeAktualisierenAnfrage(data);

			case ANLEGEN:
				return erzeugeAnlegenAnfrage(data);

			case LESEN:
				return erzeugeLesenAnfrage(data);

			default:
				return erzeugeUngueltigeAnfrage(data, "Ungültiges Anfragekommando " + anfrageKommando + " empfangen");
			}
		} catch (IllegalArgumentException e) {
			return erzeugeUngueltigeAnfrage(data, e.getLocalizedMessage());
		}
	}

	private GlUngueltigeAnfrage erzeugeUngueltigeAnfrage(Data data, String message) {

		String absenderZeichen = data.getTextValue("AbsenderZeichen").getText();
		MessQuerschnitt messQuerschnitt = null;

		SystemObject mqObj = data.getItem("Anfrage").getReferenceValue("Messquerschnitt").getSystemObject();
		if (mqObj != null) {
			messQuerschnitt = objektFactory.createMessQuerschnitt(mqObj);
		}

		SystemObject anfrageObj = data.getReferenceValue("absenderId").getSystemObject();
		GlUngueltigeAnfrage anfrage = GlUngueltigeAnfrage.of(objektFactory.createAnfrager(anfrageObj.getId()),
				absenderZeichen, messQuerschnitt, message);
		return anfrage;
	}

	private GlAnlegenAnfrage erzeugeAnlegenAnfrage(Data data) {
		String absenderZeichen = data.getTextValue("AbsenderZeichen").getText();
		MessQuerschnitt messQuerschnitt = null;

		SystemObject mqObj = data.getItem("Anfrage").getReferenceValue("Messquerschnitt").getSystemObject();
		if (mqObj != null) {
			messQuerschnitt = objektFactory.createMessQuerschnitt(mqObj);
		}

		SystemObject anfrageObj = data.getReferenceValue("absenderId").getSystemObject();

		List<MqGanglinie> ganglinien = new ArrayList<>();
		Array array = data.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien");
		for (int idx = 0; idx < array.getLength(); idx++) {
			ganglinien.add(createGanglinie(messQuerschnitt, array.getItem(idx)));
		}
		return GlAnlegenAnfrage.of(objektFactory.createAnfrager(anfrageObj.getId()), absenderZeichen, ganglinien);

	}

	private GlAktualisierenAnfrage erzeugeAktualisierenAnfrage(Data data) {
		String absenderZeichen = data.getTextValue("AbsenderZeichen").getText();
		MessQuerschnitt messQuerschnitt = null;

		SystemObject mqObj = data.getItem("Anfrage").getReferenceValue("Messquerschnitt").getSystemObject();
		if (mqObj != null) {
			messQuerschnitt = objektFactory.createMessQuerschnitt(mqObj);
		}

		SystemObject anfrageObj = data.getReferenceValue("absenderId").getSystemObject();

		List<MqGanglinie> ganglinien = new ArrayList<>();
		Array array = data.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien");
		for (int idx = 0; idx < array.getLength(); idx++) {
			ganglinien.add(createGanglinie(messQuerschnitt, array.getItem(idx)));
		}
		return GlAktualisierenAnfrage.of(objektFactory.createAnfrager(anfrageObj.getId()), absenderZeichen, ganglinien);
	}

	private GlLoeschenAnfrage erzeugeLoeschenAnfrage(Data data) {

		String absenderZeichen = data.getTextValue("AbsenderZeichen").getText();
		MessQuerschnitt messQuerschnitt = null;

		SystemObject mqObj = data.getItem("Anfrage").getReferenceValue("Messquerschnitt").getSystemObject();
		if (mqObj != null) {
			messQuerschnitt = objektFactory.createMessQuerschnitt(mqObj);
		}

		ReferenceArray ereignisTypArray = data.getItem("Anfrage").getReferenceArray("EreignisTyp");
		Collection<EreignisTyp> ereignisTypen = new ArrayList<>();
		for (int idx = 0; idx < ereignisTypArray.getLength(); idx++) {
			String pid = ereignisTypArray.getReferenceValue(idx).getSystemObjectPid();
			if (pid.isEmpty()) {
				continue;
			}
			ereignisTypen.add(EreignisTyp.of(pid));
		}

		NumberArray ganglinienIdArray = data.getItem("Anfrage").getUnscaledArray("GanglinienId");
		Collection<Long> ganglinienId = new ArrayList<>();
		for (int idx = 0; idx < ganglinienIdArray.getLength(); idx++) {
			ganglinienId.add(ganglinienIdArray.getValue(idx).longValue());
		}

		SystemObject anfrageObj = data.getReferenceValue("absenderId").getSystemObject();
		GlLoeschenAnfrage anfrage = GlLoeschenAnfrage.of(objektFactory.createAnfrager(anfrageObj.getId()),
				absenderZeichen, messQuerschnitt, ereignisTypen, ganglinienId);
		return anfrage;
	}

	private GlSchreibenAnfrage erzeugeSchreibenAnfrage(Data data) {

		String absenderZeichen = data.getTextValue("AbsenderZeichen").getText();
		MessQuerschnitt messQuerschnitt = null;

		SystemObject mqObj = data.getItem("Anfrage").getReferenceValue("Messquerschnitt").getSystemObject();
		if (mqObj != null) {
			messQuerschnitt = objektFactory.createMessQuerschnitt(mqObj);
		}

		SystemObject anfrageObj = data.getReferenceValue("absenderId").getSystemObject();

		List<MqGanglinie> ganglinien = new ArrayList<>();
		Array array = data.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien");
		for (int idx = 0; idx < array.getLength(); idx++) {
			ganglinien.add(createGanglinie(messQuerschnitt, array.getItem(idx)));
		}
		return GlSchreibenAnfrage.of(objektFactory.createAnfrager(anfrageObj.getId()), absenderZeichen, ganglinien);
	}

	private MqGanglinie createGanglinie(MessQuerschnitt messQuerschnitt, Data data) {

		EreignisTyp ereignisTyp = null;
		SystemObject ereignisObj = data.getReferenceValue("EreignisTyp").getSystemObject();
		if (ereignisObj != null) {
			ereignisTyp = EreignisTyp.of(ereignisObj.getPid());
		}

		long ganglinieId = data.getUnscaledValue("GanglinieId").longValue();
		MqGanglinie ganglinie = new MqGanglinie(ganglinieId, messQuerschnitt, ereignisTyp);

		// Verfahren
		ganglinie.setBSplineOrdnung((int) data.getUnscaledValue("Ordnung").longValue());
		ganglinie.setReferenz(data.getUnscaledValue("Referenzganglinie").getState().getName().equalsIgnoreCase("ja"));
		ganglinie.setAnzahlVerschmelzungen((int) data.getUnscaledValue("AnzahlVerschmelzungen").longValue());
		ganglinie.setLetzteVerschmelzung(LocalDateTime.ofInstant(
				Instant.ofEpochMilli(data.getTimeValue("LetzteVerschmelzung").getMillis()), ZoneId.systemDefault()));
		ganglinie.setTyp(GanglinieTyp.with(data.getUnscaledValue("GanglinienTyp").intValue()));
		ganglinie.setApproximationDaK(ApproximationTyp.with(data.getUnscaledValue("GanglinienVerfahren").intValue()));

		// Stützstellen
		Array stuetzStellenArray = data.getArray("Stützstelle");
		for (int j = 0; j < stuetzStellenArray.getLength(); j++) {

			Data stuetzstellenItem = stuetzStellenArray.getItem(j);
			long zeitstempel = stuetzstellenItem.getTimeValue("Zeit").getMillis();

			Double qKfz0 = null;
			Double qLkw0 = null;
			Double vPkw0 = null;
			Double vLkw0 = null;

			if (Double.compare(stuetzstellenItem.getScaledValue("QKfz").doubleValue(), Messwerte.UNDEFINIERT) != 0) {
				qKfz0 = stuetzstellenItem.getScaledValue("QKfz").doubleValue();
			}
			if (Double.compare(stuetzstellenItem.getScaledValue("QLkw").doubleValue(), Messwerte.UNDEFINIERT) != 0) {
				qLkw0 = stuetzstellenItem.getScaledValue("QLkw").doubleValue();
			}
			if (Double.compare(stuetzstellenItem.getScaledValue("VPkw").doubleValue(), Messwerte.UNDEFINIERT) != 0) {
				vPkw0 = stuetzstellenItem.getScaledValue("VPkw").doubleValue();
			}
			if (Double.compare(stuetzstellenItem.getScaledValue("VLkw").doubleValue(), Messwerte.UNDEFINIERT) != 0) {
				vLkw0 = stuetzstellenItem.getScaledValue("VLkw").doubleValue();
			}
			ganglinie.aktualisiereMesswert(zeitstempel, new Messwerte(qKfz0, qLkw0, vPkw0, vLkw0));
		}

		return ganglinie;
	}

	private GlLesenAnfrage erzeugeLesenAnfrage(Data data) {

		String absenderZeichen = data.getTextValue("AbsenderZeichen").getText();
		MessQuerschnitt messQuerschnitt = null;

		SystemObject mqObj = data.getItem("Anfrage").getReferenceValue("Messquerschnitt").getSystemObject();
		if (mqObj != null) {
			messQuerschnitt = objektFactory.createMessQuerschnitt(mqObj);
		}

		ReferenceArray ereignisTypArray = data.getItem("Anfrage").getReferenceArray("EreignisTyp");
		Collection<EreignisTyp> ereignisTypen = new ArrayList<>();
		for (int idx = 0; idx < ereignisTypArray.getLength(); idx++) {
			String pid = ereignisTypArray.getReferenceValue(idx).getSystemObjectPid();
			if (pid.isEmpty()) {
				continue;
			}
			ereignisTypen.add(EreignisTyp.of(pid));
		}

		SystemObject anfrageObj = data.getReferenceValue("absenderId").getSystemObject();
		GlLesenAnfrage anfrage = GlLesenAnfrage.of(objektFactory.createAnfrager(anfrageObj.getId()), absenderZeichen,
				messQuerschnitt, ereignisTypen);
		return anfrage;
	}
}
