/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.1 Ganglinienprognose
 * Copyright (C) 2019 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.glprog.glprog.portals;

import java.text.MessageFormat;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlProgAnfrage;
import de.bsvrz.iav.gllib.gllib.portals.DavObjektFactory;
import de.bsvrz.iav.glprog.glprog.AnfrageInBearbeitung;
import de.bsvrz.iav.glprog.glprog.GlProg;
import de.bsvrz.iav.glprog.glprog.domain.comm.GanglinienPrognoseAnfrageServer;
import de.bsvrz.iav.glprog.glprog.domain.comm.PrognoseAnfrageEmpfaenger;
import de.bsvrz.iav.glprog.glprog.domain.comm.PrognoseAntwortSender;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nimmt Anfragen vom Datenverteiler entgegen und startet deren Bearbeitung.
 *
 * @author BitCtrl Systems GmbH, Schumann
 */
public class DavGanglinienPrognoseAnfrageServer implements GanglinienPrognoseAnfrageServer {

    /** Der Logger dieser Klasse. */
    private static final Debug LOGGER = Debug.getLogger();

    private final ExecutorService threadPool;
    private ClientDavInterface dav;

    /**
     * Initialisiert die eigentliche Anwendungslogik.
     *
     * @param applikation die Ganglinienprognose-Applikation.
     * @param objektFactory die Objektfactory zum Erstellen von Datenverteilerobjekten oder deren Entsprechungen
     * @param maxAnfragen die maximale Anzahl von Anfragen, die gleichzeitig (als
     *                    Thread) bearbeitet werden sollen.
     */
    public DavGanglinienPrognoseAnfrageServer(GlProg applikation, DavObjektFactory objektFactory, final int maxAnfragen) {
        this.dav = objektFactory.getDav();
        threadPool = Executors.newFixedThreadPool(maxAnfragen);
        PrognoseAnfrageEmpfaenger anfrageEmpfaenger = new DavPrognoseAnfrageEmpfaenger(objektFactory);
        anfrageEmpfaenger.anfrageEmpfangen().addHandler(anfrage -> bearbeiteAnfrage(applikation, anfrage));

        LOGGER.config(() -> MessageFormat.format("Verwaltung bereit. Bearbeite maximal {0} Anfragen gleichzeitig.",
                maxAnfragen));
    }

    /**
     * Empfängt neue Anfragen und startet deren Bearbeitung.
     * 
     * @param anfrage die zu bearbeitende Anfrage
     */
    @Override
    public void bearbeiteAnfrage(GlProg applikation, GlProgAnfrage anfrage) {
        PrognoseAntwortSender antwortSender = new DavPrognoseAntwortSender(dav, anfrage.getAnfrager());
        threadPool.execute(new AnfrageInBearbeitung(applikation, anfrage, antwortSender));
    }

    @Override
    public void stop() {
        // Abmeldung beim Datenverteiler wird ignoriert
    }

}
